"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AIEditorContribution = void 0;
const tslib_1 = require("tslib");
const debounce_1 = tslib_1.__importDefault(require("lodash/debounce"));
const di_1 = require("@opensumi/di");
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const browser_1 = require("@opensumi/ide-core-browser/lib/menu/next/renderer/ctxmenu/browser");
const ide_core_common_1 = require("@opensumi/ide-core-common");
const menu_service_1 = require("@opensumi/ide-design/lib/browser/override/menu.service");
const browser_2 = require("@opensumi/ide-editor/lib/browser");
const monaco_api_1 = require("@opensumi/ide-monaco/lib/browser/monaco-api");
const common_1 = require("../common");
const ai_native_service_1 = require("./ai-native.service");
const completeProvider_1 = require("./inline-completions/completeProvider");
const ai_completions_service_1 = require("./inline-completions/service/ai-completions.service");
const parser_1 = require("./languages/parser");
const rename_service_1 = require("./rename/rename.service");
const types_1 = require("./types");
const inline_chat_feature_registry_1 = require("./widget/inline-chat/inline-chat.feature.registry");
const inline_chat_service_1 = require("./widget/inline-chat/inline-chat.service");
const inline_content_widget_1 = require("./widget/inline-chat/inline-content-widget");
const inline_diff_widget_1 = require("./widget/inline-diff/inline-diff-widget");
let AIEditorContribution = class AIEditorContribution extends ide_core_common_1.Disposable {
    constructor() {
        super();
        this.aiInlineChatDisposed = new ide_core_common_1.Disposable();
        this.aiInlineChatOperationDisposed = new ide_core_common_1.Disposable();
        this.inlineChatInUsing = false;
        this.logger = this.loggerManagerClient.getLogger(ide_core_common_1.SupportLogNamespace.Browser);
    }
    disposeAllWidget() {
        [
            this.aiDiffWidget,
            this.aiInlineContentWidget,
            this.aiInlineChatDisposed,
            this.aiInlineChatOperationDisposed,
        ].forEach((widget) => {
            widget === null || widget === void 0 ? void 0 : widget.dispose();
        });
        this.aiNativeService.cancelToken();
        this.inlineChatInUsing = false;
    }
    contribute(editor) {
        if (!editor) {
            return this;
        }
        const { monacoEditor, currentUri } = editor;
        if (currentUri && currentUri.codeUri.scheme !== ide_core_common_1.Schemes.file) {
            return this;
        }
        this.registerLanguageFeatures(editor);
        this.disposables.push(this.aiNativeService.onInlineChatVisible((value) => {
            if (value) {
                this.showInlineChat(editor);
            }
            else {
                this.disposeAllWidget();
            }
        }));
        let needShowInlineChat = false;
        this.disposables.push(monacoEditor.onMouseDown(() => {
            needShowInlineChat = false;
        }), monacoEditor.onMouseUp((event) => {
            const target = event.target;
            const detail = target.detail;
            if (detail && typeof detail === 'string' && detail === common_1.AIInlineChatContentWidget) {
                needShowInlineChat = false;
            }
            else {
                needShowInlineChat = true;
            }
        }), monacoEditor.onDidChangeModelContent((e) => {
            const changes = e.changes;
            for (const change of changes) {
                if (change.text === '') {
                    this.aiInlineCompletionsProvider.isDelEvent = true;
                    this.aiInlineCompletionsProvider.cancelRequest();
                }
                else {
                    this.aiInlineCompletionsProvider.isDelEvent = false;
                }
            }
        }), monacoEditor.onWillChangeModel(() => {
            this.aiCompletionsService.hideStatusBarItem();
            this.disposeAllWidget();
        }), monacoEditor.onDidBlurEditorText(() => {
            this.aiCompletionsService.hideStatusBarItem();
            this.aiCompletionsService.setVisibleCompletion(false);
        }), monacoEditor.onDidScrollChange(() => {
            /**
             * 其他的 ctxmenu 服务注册的菜单在 onHide 函数里会有其他逻辑处理，例如在 editor.context.ts 会在 hide 的时候 focus 编辑器，影响使用
             */
            this.ctxMenuRenderer.onHide = undefined;
            this.ctxMenuRenderer.hide(true);
        }));
        let prefInlineChatAutoVisible = this.preferenceService.getValid(ide_core_common_1.AINativeSettingSectionsId.INLINE_CHAT_AUTO_VISIBLE, true);
        this.disposables.push({
            dispose: () => {
                this.preferenceService.onSpecificPreferenceChange(ide_core_common_1.AINativeSettingSectionsId.INLINE_CHAT_AUTO_VISIBLE, ({ newValue }) => {
                    prefInlineChatAutoVisible = newValue;
                });
            },
        });
        this.disposables.push(ide_core_common_1.Event.debounce(ide_core_common_1.Event.any(monacoEditor.onDidChangeCursorSelection, monacoEditor.onMouseUp), (_, e) => e, 100)(() => {
            if (!prefInlineChatAutoVisible) {
                return;
            }
            if (!needShowInlineChat) {
                return;
            }
            if (this.aiInlineChatService.status !== 0 /* EInlineChatStatus.READY */ &&
                this.aiInlineChatService.status !== 3 /* EInlineChatStatus.ERROR */) {
                return;
            }
            this.showInlineChat(editor);
        }));
        // 判断用户是否选择了一块区域或者移动光标 取消掉请补全求
        const selectionChange = () => {
            this.aiCompletionsService.hideStatusBarItem();
            const selection = monacoEditor.getSelection();
            // 判断是否选中区域
            if (selection.startLineNumber !== selection.endLineNumber || selection.startColumn !== selection.endColumn) {
                this.aiInlineCompletionsProvider.cancelRequest();
            }
            requestAnimationFrame(() => {
                this.aiCompletionsService.setVisibleCompletion(false);
            });
        };
        const debouncedSelectionChange = (0, debounce_1.default)(() => selectionChange(), 50, {
            maxWait: 200,
            leading: true,
            trailing: true,
        });
        this.disposables.push(this.eventBus.on(browser_2.EditorSelectionChangeEvent, (e) => {
            if (e.payload.source === 'mouse') {
                debouncedSelectionChange();
            }
            else {
                debouncedSelectionChange.cancel();
                selectionChange();
            }
        }));
        return this;
    }
    async showInlineChat(editor, actionId) {
        var _a;
        if (!this.aiNativeConfigService.capabilities.supportsInlineChat) {
            return;
        }
        if (this.inlineChatInUsing) {
            return;
        }
        this.inlineChatInUsing = true;
        this.disposeAllWidget();
        const { monacoEditor, currentUri } = editor;
        if (!currentUri || currentUri.codeUri.scheme !== ide_core_common_1.Schemes.file) {
            return;
        }
        const selection = monacoEditor.getSelection();
        const selectCode = selection && ((_a = monacoEditor.getModel()) === null || _a === void 0 ? void 0 : _a.getValueInRange(selection));
        if (!selection || !(selectCode === null || selectCode === void 0 ? void 0 : selectCode.trim())) {
            this.disposeAllWidget();
            return;
        }
        this.aiInlineChatService.launchChatStatus(0 /* EInlineChatStatus.READY */);
        this.aiInlineContentWidget = this.injector.get(inline_content_widget_1.AIInlineContentWidget, [monacoEditor]);
        this.aiInlineContentWidget.show({
            selection,
        });
        this.aiInlineChatDisposed.addDispose(this.aiInlineContentWidget.onActionClick((id) => {
            this.runInlineChatAction(id, monacoEditor);
        }));
        if (actionId) {
            this.aiInlineContentWidget.clickActionId(actionId);
        }
    }
    async runInlineChatAction(id, monacoEditor) {
        const handler = this.inlineChatFeatureRegistry.getEditorHandler(id);
        const action = this.inlineChatFeatureRegistry.getAction(id);
        if (!handler || !action) {
            return;
        }
        const selection = monacoEditor.getSelection();
        if (!selection) {
            this.logger.error('No selection found, aborting inline chat action.');
            return;
        }
        const { execute, providerDiffPreviewStrategy } = handler;
        if (execute) {
            execute(monacoEditor);
            this.disposeAllWidget();
        }
        if (providerDiffPreviewStrategy) {
            const crossSelection = selection
                .setStartPosition(selection.startLineNumber, 1)
                .setEndPosition(selection.endLineNumber, Number.MAX_SAFE_INTEGER);
            const relationId = this.aiReporter.start(action.name, { message: action.name });
            const startTime = +new Date();
            const result = await this.handleDiffPreviewStrategy(monacoEditor, providerDiffPreviewStrategy, crossSelection, relationId);
            this.aiReporter.end(relationId, { message: result, success: !!result, replytime: +new Date() - startTime });
            this.aiInlineChatDisposed.addDispose(this.aiInlineChatService.onDiscard(() => {
                this.aiReporter.end(relationId, { message: result, success: !!result, isDrop: true });
                this.disposeAllWidget();
            }));
            this.aiInlineChatDisposed.addDispose(this.aiInlineChatService.onRegenerate(async () => {
                const retryStartTime = +new Date();
                const retryResult = await this.handleDiffPreviewStrategy(monacoEditor, providerDiffPreviewStrategy, crossSelection, relationId);
                this.aiReporter.end(relationId, {
                    message: retryResult,
                    success: !!result,
                    replytime: +new Date() - retryStartTime,
                    isRetry: true,
                });
            }));
        }
    }
    async handleDiffPreviewStrategy(monacoEditor, strategy, crossSelection, relationId) {
        const model = monacoEditor.getModel();
        if (!model || !crossSelection) {
            return;
        }
        this.resetDiffEnvironment();
        const crossCode = model.getValueInRange(crossSelection);
        this.aiInlineChatService.launchChatStatus(1 /* EInlineChatStatus.THINKING */);
        const response = await strategy(monacoEditor, this.aiNativeService.cancelIndicator.token);
        if (this.aiInlineChatDisposed.disposed || ide_core_common_1.CancelResponse.is(response)) {
            this.aiInlineChatService.launchChatStatus(0 /* EInlineChatStatus.READY */);
            return;
        }
        if (ide_core_common_1.ErrorResponse.is(response)) {
            this.aiInlineChatService.launchChatStatus(3 /* EInlineChatStatus.ERROR */);
            return;
        }
        this.aiInlineChatService.launchChatStatus(2 /* EInlineChatStatus.DONE */);
        let answer = this.extractAnswerFromResponse(response);
        if (!answer) {
            return;
        }
        answer = this.formatAnswer(answer, crossCode);
        this.visibleDiffWidget(monacoEditor, crossSelection, answer);
        this.aiInlineChatOperationDisposed.addDispose([
            this.aiInlineChatService.onAccept(() => {
                var _a;
                this.aiReporter.end(relationId, { message: 'accept', success: true, isReceive: true });
                (_a = monacoEditor.getModel()) === null || _a === void 0 ? void 0 : _a.pushEditOperations(null, [{ range: crossSelection, text: answer }], () => null);
                (0, ide_core_common_1.runWhenIdle)(() => {
                    this.disposeAllWidget();
                });
            }),
            this.aiInlineChatService.onThumbs((isLike) => {
                this.aiReporter.end(relationId, { isLike });
            }),
            this.aiDiffWidget.onMaxLincCount((count) => {
                requestAnimationFrame(() => {
                    if (crossSelection.endLineNumber === model.getLineCount()) {
                        const lineHeight = monacoEditor.getOption(monaco_api_1.monaco.editor.EditorOption.lineHeight);
                        this.aiInlineContentWidget.offsetTop(lineHeight * count + 12);
                    }
                });
            }),
        ]);
        return answer;
    }
    resetDiffEnvironment() {
        var _a;
        (_a = this.aiDiffWidget) === null || _a === void 0 ? void 0 : _a.dispose();
        this.aiInlineChatOperationDisposed.dispose();
    }
    extractAnswerFromResponse(response) {
        const regex = /```\w*([\s\S]+?)\s*```/;
        const match = regex.exec(response.message);
        return match ? match[1].trim() : response.message.trim();
    }
    formatAnswer(answer, crossCode) {
        const leadingWhitespaceMatch = crossCode.match(/^\s*/);
        const indent = leadingWhitespaceMatch ? leadingWhitespaceMatch[0] : '  ';
        return answer
            .split('\n')
            .map((line, index) => (index === 0 ? line : `${indent}${line}`))
            .join('\n');
    }
    visibleDiffWidget(monacoEditor, crossSelection, answer) {
        monacoEditor.setHiddenAreas([crossSelection], inline_diff_widget_1.AIDiffWidget._hideId);
        this.aiDiffWidget = this.injector.get(inline_diff_widget_1.AIDiffWidget, [monacoEditor, crossSelection, answer]);
        this.aiDiffWidget.create();
        this.aiDiffWidget.showByLine(crossSelection.startLineNumber - 1, crossSelection.endLineNumber - crossSelection.startLineNumber + 2);
        this.updateInlineContentWidgetPosition(crossSelection);
    }
    updateInlineContentWidgetPosition(crossSelection) {
        var _a, _b;
        (_a = this.aiInlineContentWidget) === null || _a === void 0 ? void 0 : _a.setOptions({
            position: {
                lineNumber: crossSelection.endLineNumber + 1,
                column: 1,
            },
        });
        (_b = this.aiInlineContentWidget) === null || _b === void 0 ? void 0 : _b.layoutContentWidget();
    }
    async registerLanguageFeatures(editor) {
        const { monacoEditor, currentUri } = editor;
        if (currentUri && currentUri.codeUri.scheme !== ide_core_common_1.Schemes.file) {
            return;
        }
        this.disposables.push(ide_core_common_1.Event.debounce(monacoEditor.onWillChangeModel, (_, e) => e, 300)(async () => {
            if (this.modelSessionDisposable) {
                this.modelSessionDisposable.dispose();
            }
            const model = monacoEditor.getModel();
            if (!model) {
                return;
            }
            this.modelSessionDisposable = new ide_core_common_1.Disposable();
            if (this.aiNativeConfigService.capabilities.supportsInlineCompletion) {
                this.contributions.getContributions().forEach((contribution) => {
                    if (contribution.middleware) {
                        this.latestMiddlewareCollector = contribution.middleware;
                    }
                });
                this.aiInlineCompletionsProvider.registerEditor(editor);
                this.modelSessionDisposable.addDispose({
                    dispose: () => {
                        this.aiInlineCompletionsProvider.dispose();
                    },
                });
                this.modelSessionDisposable.addDispose(monaco_api_1.monaco.languages.registerInlineCompletionsProvider(model.getLanguageId(), {
                    provideInlineCompletions: async (model, position, context, token) => {
                        var _a, _b, _c, _d;
                        if ((_b = (_a = this.latestMiddlewareCollector) === null || _a === void 0 ? void 0 : _a.language) === null || _b === void 0 ? void 0 : _b.provideInlineCompletions) {
                            this.aiCompletionsService.setMiddlewareComplete((_d = (_c = this.latestMiddlewareCollector) === null || _c === void 0 ? void 0 : _c.language) === null || _d === void 0 ? void 0 : _d.provideInlineCompletions);
                        }
                        const list = await this.aiInlineCompletionsProvider.provideInlineCompletionItems(model, position, context, token);
                        this.logger.log('provideInlineCompletions: ', list.items.map((data) => data.insertText));
                        return list;
                    },
                    freeInlineCompletions() { },
                    handleItemDidShow: (completions, item) => {
                        if (completions.items.length > 0) {
                            this.aiCompletionsService.setVisibleCompletion(true);
                        }
                    },
                }));
            }
            if (this.aiNativeConfigService.capabilities.supportsRenameSuggestions) {
                const provider = async (model, range, token) => {
                    const result = await this.renameSuggestionService.provideRenameSuggestions(model, range, token);
                    return result;
                };
                this.modelSessionDisposable.addDispose(monaco_api_1.monaco.languages.registerNewSymbolNameProvider(model.getLanguageId(), {
                    provideNewSymbolNames: provider,
                }));
            }
            if (this.aiNativeConfigService.capabilities.supportsInlineChat) {
                // 通过 code actions 来透出我们 inline chat 的功能
                const languageId = model.getLanguageId();
                this.modelSessionDisposable.addDispose(this.inlineChatFeatureRegistry.onActionRun(({ id, range }) => {
                    monacoEditor.setSelection(range);
                    this.showInlineChat(editor, id);
                }));
                this.modelSessionDisposable.addDispose(monaco_api_1.monaco.languages.registerCodeActionProvider(languageId, {
                    provideCodeActions: async (model) => {
                        const parser = this.languageParserFactory(languageId);
                        if (!parser) {
                            return;
                        }
                        const cursorPosition = monacoEditor.getPosition();
                        const actions = this.inlineChatFeatureRegistry.getCodeActions();
                        const functionInfo = await parser.provideFunctionInfo(model, cursorPosition);
                        if (functionInfo) {
                            return {
                                actions: actions.map((v) => {
                                    const command = {};
                                    if (v.command) {
                                        command.id = v.command.id;
                                        command.arguments = [functionInfo.range];
                                    }
                                    return Object.assign(Object.assign({}, v), { title: v.title + ` for Function: ${functionInfo.name}`, ranges: [functionInfo.range], command });
                                }),
                                dispose() { },
                            };
                        }
                        const codeblock = await parser.findCodeBlock(model, cursorPosition);
                        if (codeblock) {
                            return {
                                actions: actions.map((v) => {
                                    const command = {};
                                    if (v.command) {
                                        command.id = v.command.id;
                                        command.arguments = [codeblock.range];
                                    }
                                    return Object.assign(Object.assign({}, v), { title: v.title, ranges: [codeblock.range], command });
                                }),
                                dispose() { },
                            };
                        }
                    },
                }));
            }
        }));
    }
    dispose() {
        super.dispose();
        if (this.modelSessionDisposable) {
            this.modelSessionDisposable.dispose();
        }
    }
};
tslib_1.__decorate([
    (0, di_1.Autowired)(di_1.INJECTOR_TOKEN),
    tslib_1.__metadata("design:type", di_1.Injector)
], AIEditorContribution.prototype, "injector", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.AINativeConfigService),
    tslib_1.__metadata("design:type", ide_core_browser_1.AINativeConfigService)
], AIEditorContribution.prototype, "aiNativeConfigService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.IAIInlineChatService),
    tslib_1.__metadata("design:type", inline_chat_service_1.AIInlineChatService)
], AIEditorContribution.prototype, "aiInlineChatService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ai_native_service_1.AINativeService),
    tslib_1.__metadata("design:type", ai_native_service_1.AINativeService)
], AIEditorContribution.prototype, "aiNativeService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_1.ILoggerManagerClient),
    tslib_1.__metadata("design:type", Object)
], AIEditorContribution.prototype, "loggerManagerClient", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.PreferenceService),
    tslib_1.__metadata("design:type", Object)
], AIEditorContribution.prototype, "preferenceService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(browser_1.IBrowserCtxMenu),
    tslib_1.__metadata("design:type", menu_service_1.DesignBrowserCtxMenuService)
], AIEditorContribution.prototype, "ctxMenuRenderer", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_1.InlineChatFeatureRegistryToken),
    tslib_1.__metadata("design:type", inline_chat_feature_registry_1.InlineChatFeatureRegistry)
], AIEditorContribution.prototype, "inlineChatFeatureRegistry", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_1.IAIReporter),
    tslib_1.__metadata("design:type", Object)
], AIEditorContribution.prototype, "aiReporter", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(types_1.AINativeCoreContribution),
    tslib_1.__metadata("design:type", Object)
], AIEditorContribution.prototype, "contributions", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(completeProvider_1.AIInlineCompletionsProvider),
    tslib_1.__metadata("design:type", completeProvider_1.AIInlineCompletionsProvider)
], AIEditorContribution.prototype, "aiInlineCompletionsProvider", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(rename_service_1.RenameSuggestionsService),
    tslib_1.__metadata("design:type", rename_service_1.RenameSuggestionsService)
], AIEditorContribution.prototype, "renameSuggestionService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ai_completions_service_1.AICompletionsService),
    tslib_1.__metadata("design:type", ai_completions_service_1.AICompletionsService)
], AIEditorContribution.prototype, "aiCompletionsService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_1.IEventBus),
    tslib_1.__metadata("design:type", Object)
], AIEditorContribution.prototype, "eventBus", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(parser_1.LanguageParserFactory),
    tslib_1.__metadata("design:type", Function)
], AIEditorContribution.prototype, "languageParserFactory", void 0);
AIEditorContribution = tslib_1.__decorate([
    (0, di_1.Injectable)(),
    tslib_1.__metadata("design:paramtypes", [])
], AIEditorContribution);
exports.AIEditorContribution = AIEditorContribution;
//# sourceMappingURL=ai-editor.contribution.js.map